﻿using HarmonyLib;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using UnityEngine;
using Verse;

namespace RIMMSqol
{
    /**
     * If we detect a packAnimal we test if the pack graphic is available. If not we change the packAnimal flag for the regular call to avoid errors about missing graphics
     * and reset the flag after the graphics have been resolved.
     */
    [HarmonyPatch(typeof(PawnGraphicSet))]
    [HarmonyPatch("ResolveAllGraphics")]
    [HarmonyPatchNamespace("RIMMSQoL.PawnPackRendering")]
    public static class PawnGraphicSet_ResolveAllGraphics
    {
        private static Dictionary<string, bool> hasPackGraphics = new Dictionary<string, bool>();

        [HarmonyPriority(Priority.Last)]
        static void Prefix(PawnGraphicSet __instance, out bool __state)
        {
            if (__instance.pawn.RaceProps.Humanlike)
            {
                __state = false;
                return;
            }
            if (__instance.pawn.RaceProps.packAnimal)
            {
                if (!hasPackGraphics.TryGetValue(__instance.pawn.def.defName, out bool available))
                {
                    string basePath;
                    PawnKindLifeStage curKindLifeStage = __instance.pawn.ageTracker.CurKindLifeStage;
                    if (curKindLifeStage == null)
                    {
                        __state = false;
                        return;
                    }
                    if (__instance.pawn.gender != Gender.Female || curKindLifeStage.femaleGraphicData == null)
                    {
                        basePath = curKindLifeStage.bodyGraphicData.Graphic.path;
                    }
                    else
                    {
                        basePath = curKindLifeStage.femaleGraphicData.Graphic.path;
                    }
                    basePath += "Pack";
                    if (ContentFinder<Texture2D>.Get(basePath, false) != null || ContentFinder<Texture2D>.Get(basePath + "_north", false) != null || 
                        ContentFinder<Texture2D>.Get(basePath + "_east", false) != null || ContentFinder<Texture2D>.Get(basePath + "_south", false) != null || 
                        ContentFinder<Texture2D>.Get(basePath + "_west", false) != null)
                    {
                        hasPackGraphics.Add(__instance.pawn.def.defName, available = true);
                    }
                    else
                    {
                        hasPackGraphics.Add(__instance.pawn.def.defName, available = false);
                    }
                }
                if (available)
                {
                    __state = false;
                    return;
                }
                else
                {
                    __instance.pawn.RaceProps.packAnimal = false;
                    __state = true;
                    return;
                }
            }
            __state = false;
        }

        static void Postfix(PawnGraphicSet __instance, bool __state)
        {
            if (__state)
            {
                __instance.pawn.RaceProps.packAnimal = true;
            }
        }
    }
}
